# -*- coding: utf-8 -*-
# cython: language_level=3, always_allow_keywords=True

## Copyright 1999-2014 by LivingLogic AG, Bayreuth/Germany
## Copyright 1999-2014 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/xist/__init__.py for the license


"""
This is a namespace module implementing `SVG 1.1`__.

__ http://www.w3.org/TR/SVG11/
"""


from ll.xist import xsc, sims


__docformat__ = "reStructuredText"


xmlns = "http://www.w3.org/2000/svg"


class DocTypeSVG11(xsc.DocType):
	"""
	document type for SVG 1.1
	"""
	def __init__(self):
		xsc.DocType.__init__(self, 'svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd"')


class alignment_baseline(xsc.Attrs):
	class alignment_baseline(xsc.TextAttr):
		xmlname = "alignment-baseline"
		values = ("auto", "baseline", "before-edge", "text-before-edge", "middle", "central", "after-edge", "text-after-edge", "ideographic", "alphabetic", "hanging", "mathematical", "inherit")


class baseline_shift(xsc.Attrs):
	class baseline_shift(xsc.TextAttr): xmlname = "baseline-shift"


class class_(xsc.Attrs):
	class class_(xsc.TextAttr): xmlname = "class"


class clip(xsc.Attrs):
	class clip(xsc.TextAttr): pass


class clip_path(xsc.Attrs):
	class clip_path(xsc.TextAttr): xmlname = "clip-path"


class clip_rule(xsc.Attrs):
	class clip_rule(xsc.TextAttr):
		xmlname = "clip-rule"
		values = ("nonzero", "evenodd", "inherit")


class color(xsc.Attrs):
	class color(xsc.ColorAttr): pass


class color_interpolation(xsc.Attrs):
	class color_interpolation(xsc.TextAttr):
		xmlname = "color-interpolation"
		values = ("auto", "sRGB", "linearRGB", "inherit")


class color_interpolation_filters(xsc.Attrs):
	class color_interpolation_filters(xsc.TextAttr):
		xmlname = "color-interpolation-filters"
		values = ("auto", "sRGB", "linearRGB", "inherit")


class color_profile2(xsc.Attrs):
	class color_profile(xsc.TextAttr): xmlname = "color-profile"


class color_rendering(xsc.Attrs):
	class color_rendering(xsc.TextAttr):
		xmlname = "color-rendering"
		values = ("auto", "optimizeSpeed", "optimizeQuality", "inherit")


class cursor2(xsc.Attrs):
	class cursor(xsc.TextAttr): pass


class direction(xsc.Attrs):
	class direction(xsc.TextAttr): values = ("ltr", "rtl", "inherit")


class display(xsc.Attrs):
	class display(xsc.TextAttr): values = ("inline", "block", "list-item", "run-in", "compact", "marker", "table", "inline-table", "table-row-group", "table-header-group", "table-footer-group", "table-row", "table-column-group", "table-column", "table-cell", "table-caption", "none", "inherit")


class dominant_baseline(xsc.Attrs):
	class dominant_baseline(xsc.TextAttr):
		xmlname = "dominant-baseline"
		values = ("auto", "use-script", "no-change", "reset-size", "ideographic", "alphabetic", "hanging", "mathematical", "central", "middle", "text-after-edge", "text-before-edge", "inherit")


class enable_background(xsc.Attrs):
	class enable_background(xsc.TextAttr): xmlname = "enable-background"


class externalResourcesRequired(xsc.Attrs):
	class externalResourcesRequired(xsc.TextAttr): values = ("false", "true")


class fill(xsc.Attrs):
	class fill(xsc.ColorAttr): pass


class fill_opacity(xsc.Attrs):
	class fill_opacity(xsc.TextAttr): xmlname = "fill-opacity"


class fill_rule(xsc.Attrs):
	class fill_rule(xsc.TextAttr):
		xmlname = "fill-rule"
		values = ("nonzero", "evenodd", "inherit")


class filter2(xsc.Attrs):
	class filter(xsc.TextAttr): pass


class flood_color(xsc.Attrs):
	class flood_color(xsc.ColorAttr): xmlname = "flood-color"


class flood_opacity(xsc.Attrs):
	class flood_opacity(xsc.TextAttr): xmlname = "flood-opacity"


class font_family(xsc.Attrs):
	class font_family(xsc.TextAttr): xmlname = "font-family"


class font_size(xsc.Attrs):
	class font_size(xsc.TextAttr): xmlname = "font-size"


class font_size_adjust(xsc.Attrs):
	class font_size_adjust(xsc.TextAttr): xmlname = "font-size-adjust"


class font_stretch(xsc.Attrs):
	class font_stretch(xsc.TextAttr):
		xmlname = "font-stretch"
		values = ("normal", "wider", "narrower", "ultra-condensed", "extra-condensed", "condensed", "semi-condensed", "semi-expanded", "expanded", "extra-expanded", "ultra-expanded", "inherit")


class font_style(xsc.Attrs):
	class font_style(xsc.TextAttr):
		xmlname = "font-style"
		values = ("normal", "italic", "oblique", "inherit")


class font_variant(xsc.Attrs):
	class font_variant(xsc.TextAttr):
		xmlname = "font-variant"
		values = ("normal", "small-caps", "inherit")


class font_weight(xsc.Attrs):
	class font_weight(xsc.TextAttr):
		xmlname = "font-weight"
		values = ("normal", "bold", "bolder", "lighter", 100, 200, 300, 400, 500, 600, 700, 800, 900, "inherit")


class glyph_orientation_horizontal(xsc.Attrs):
	class glyph_orientation_horizontal(xsc.TextAttr): xmlname = "glyph-orientation-horizontal"


class glyph_orientation_vertical(xsc.Attrs):
	class glyph_orientation_vertical(xsc.TextAttr): xmlname = "glyph-orientation-vertical"


class id(xsc.Attrs):
	class id(xsc.IDAttr): pass


class image_rendering(xsc.Attrs):
	class image_rendering(xsc.TextAttr):
		xmlname = "image-rendering"
		values = ("auto", "optimizeSpeed", "optimizeQuality", "inherit")


class kerning(xsc.Attrs):
	class kerning(xsc.TextAttr): pass


class letter_spacing(xsc.Attrs):
	class letter_spacing(xsc.TextAttr): xmlname = "letter-spacing"


class lighting_color(xsc.Attrs):
	class lighting_color(xsc.ColorAttr): xmlname = "lighting-color"


class marker_end(xsc.Attrs):
	class marker_end(xsc.TextAttr): xmlname = "marker-end"


class marker_mid(xsc.Attrs):
	class marker_mid(xsc.TextAttr): xmlname = "marker-mid"


class marker_start(xsc.Attrs):
	class marker_start(xsc.TextAttr): xmlname = "marker-start"


class mask2(xsc.Attrs):
	class mask(xsc.TextAttr): pass


class onactivate(xsc.Attrs):
	class onactivate(xsc.TextAttr): pass


class onclick(xsc.Attrs):
	class onclick(xsc.TextAttr): pass


class onfocusin(xsc.Attrs):
	class onfocusin(xsc.TextAttr): pass


class onfocusout(xsc.Attrs):
	class onfocusout(xsc.TextAttr): pass


class onload(xsc.Attrs):
	class onload(xsc.TextAttr): pass


class onmousedown(xsc.Attrs):
	class onmousedown(xsc.TextAttr): pass


class onmousemove(xsc.Attrs):
	class onmousemove(xsc.TextAttr): pass


class onmouseout(xsc.Attrs):
	class onmouseout(xsc.TextAttr): pass


class onmouseover(xsc.Attrs):
	class onmouseover(xsc.TextAttr): pass


class onmouseup(xsc.Attrs):
	class onmouseup(xsc.TextAttr): pass


class opacity(xsc.Attrs):
	class opacity(xsc.TextAttr): pass


class overflow(xsc.Attrs):
	class overflow(xsc.TextAttr): values = ("visible", "hidden", "scroll", "auto", "inherit")


class pointer_events(xsc.Attrs):
	class pointer_events(xsc.TextAttr):
		xmlname = "pointer-events"
		values = ("visiblePainted", "visibleFill", "visibleStroke", "visible", "painted", "fill", "stroke", "all", "none", "inherit")


class requiredExtensions(xsc.Attrs):
	class requiredExtensions(xsc.TextAttr): pass


class requiredFeatures(xsc.Attrs):
	class requiredFeatures(xsc.TextAttr): pass


class shape_rendering(xsc.Attrs):
	class shape_rendering(xsc.TextAttr):
		xmlname = "shape-rendering"
		values = ("auto", "optimizeSpeed", "crispEdges", "geometricPrecision", "inherit")


class stop_color(xsc.Attrs):
	class stop_color(xsc.ColorAttr): xmlname = "stop-color"


class stop_opacity(xsc.Attrs):
	class stop_opacity(xsc.TextAttr): xmlname = "stop-opacity"


class stroke(xsc.Attrs):
	class stroke(xsc.ColorAttr): pass


class stroke_dasharray(xsc.Attrs):
	class stroke_dasharray(xsc.TextAttr): xmlname = "stroke-dasharray"


class stroke_dashoffset(xsc.Attrs):
	class stroke_dashoffset(xsc.TextAttr): xmlname = "stroke-dashoffset"


class stroke_linecap(xsc.Attrs):
	class stroke_linecap(xsc.TextAttr):
		xmlname = "stroke-linecap"
		values = ("butt", "round", "square", "inherit")


class stroke_linejoin(xsc.Attrs):
	class stroke_linejoin(xsc.TextAttr):
		xmlname = "stroke-linejoin"
		values = ("miter", "round", "bevel", "inherit")


class stroke_miterlimit(xsc.Attrs):
	class stroke_miterlimit(xsc.TextAttr): xmlname = "stroke-miterlimit"


class stroke_opacity(xsc.Attrs):
	class stroke_opacity(xsc.TextAttr): xmlname = "stroke-opacity"


class stroke_width(xsc.Attrs):
	class stroke_width(xsc.TextAttr): xmlname = "stroke-width"


class style2(xsc.Attrs):
	class style(xsc.StyleAttr): pass


class systemLanguage(xsc.Attrs):
	class systemLanguage(xsc.TextAttr): pass


class text_anchor(xsc.Attrs):
	class text_anchor(xsc.TextAttr):
		xmlname = "text-anchor"
		values = ("start", "middle", "end", "inherit")


class text_decoration(xsc.Attrs):
	class text_decoration(xsc.TextAttr): xmlname = "text-decoration"


class text_rendering(xsc.Attrs):
	class text_rendering(xsc.TextAttr):
		xmlname = "text-rendering"
		values = ("auto", "optimizeSpeed", "optimizeLegibility", "geometricPrecision", "inherit")


class transform(xsc.Attrs):
	class transform(xsc.TextAttr): pass


class unicode_bidi(xsc.Attrs):
	class unicode_bidi(xsc.TextAttr):
		xmlname = "unicode-bidi"
		values = ("normal", "embed", "bidi-override", "inherit")


class visibility(xsc.Attrs):
	class visibility(xsc.TextAttr): values = ("visible", "hidden", "inherit")


class word_spacing(xsc.Attrs):
	class word_spacing(xsc.TextAttr): xmlname = "word-spacing"


class writing_mode(xsc.Attrs):
	class writing_mode(xsc.TextAttr):
		xmlname = "writing-mode"
		values = ("lr-tb", "rl-tb", "tb-rl", "lr", "rl", "tb", "inherit")


class dx(xsc.Attrs):
	class dx(xsc.TextAttr): pass


class dy(xsc.Attrs):
	class dy(xsc.TextAttr): pass


class format(xsc.Attrs):
	class format(xsc.TextAttr): pass


class glyphRef2(xsc.Attrs):
	class glyphRef(xsc.TextAttr): pass


class rotate(xsc.Attrs):
	class rotate(xsc.TextAttr): pass


class x(xsc.Attrs):
	class x(xsc.TextAttr): pass


class y(xsc.Attrs):
	class y(xsc.TextAttr): pass


class accumulate(xsc.Attrs):
	class accumulate(xsc.TextAttr): values = ("none", "sum")


class additive(xsc.Attrs):
	class additive(xsc.TextAttr): values = ("replace", "sum")


class attributeName(xsc.Attrs):
	class attributeName(xsc.TextAttr): required = True


class attributeType(xsc.Attrs):
	class attributeType(xsc.TextAttr): pass


class begin(xsc.Attrs):
	class begin(xsc.TextAttr): pass


class by(xsc.Attrs):
	class by(xsc.TextAttr): pass


class calcMode(xsc.Attrs):
	class calcMode(xsc.TextAttr): values = ("discrete", "linear", "paced", "spline")


class dur(xsc.Attrs):
	class dur(xsc.TextAttr): pass


class end(xsc.Attrs):
	class end(xsc.TextAttr): pass


class fill2(xsc.Attrs):
	class fill(xsc.TextAttr): values = ("remove", "freeze")


class from_(xsc.Attrs):
	class from_(xsc.TextAttr): xmlname = "from"


class keySplines(xsc.Attrs):
	class keySplines(xsc.TextAttr): pass


class keyTimes(xsc.Attrs):
	class keyTimes(xsc.TextAttr): pass


class max(xsc.Attrs):
	class max(xsc.TextAttr): pass


class min(xsc.Attrs):
	class min(xsc.TextAttr): pass


class onbegin(xsc.Attrs):
	class onbegin(xsc.TextAttr): pass


class onend(xsc.Attrs):
	class onend(xsc.TextAttr): pass


class onrepeat(xsc.Attrs):
	class onrepeat(xsc.TextAttr): pass


class repeatCount(xsc.Attrs):
	class repeatCount(xsc.TextAttr): pass


class repeatDur(xsc.Attrs):
	class repeatDur(xsc.TextAttr): pass


class restart(xsc.Attrs):
	class restart(xsc.TextAttr): values = ("always", "never", "whenNotActive")


class to(xsc.Attrs):
	class to(xsc.TextAttr): pass


class values(xsc.Attrs):
	class values_(xsc.TextAttr): xmlname = "values"


class cx(xsc.Attrs):
	class cx(xsc.TextAttr): pass


class cy(xsc.Attrs):
	class cy(xsc.TextAttr): pass


class height(xsc.Attrs):
	class height(xsc.TextAttr): pass


class in_(xsc.Attrs):
	class in_(xsc.TextAttr): xmlname = "in"


class in2(xsc.Attrs):
	class in2(xsc.TextAttr): required = True


class result(xsc.Attrs):
	class result(xsc.TextAttr): pass


class width(xsc.Attrs):
	class width(xsc.TextAttr): pass


class kernelUnitLength(xsc.Attrs):
	class kernelUnitLength(xsc.TextAttr): pass


class surfaceScale(xsc.Attrs):
	class surfaceScale(xsc.TextAttr): pass


class amplitude(xsc.Attrs):
	class amplitude(xsc.TextAttr): pass


class exponent(xsc.Attrs):
	class exponent(xsc.TextAttr): pass


class intercept(xsc.Attrs):
	class intercept(xsc.TextAttr): pass


class offset(xsc.Attrs):
	class offset(xsc.TextAttr): pass


class slope(xsc.Attrs):
	class slope(xsc.TextAttr): pass


class tableValues(xsc.Attrs):
	class tableValues(xsc.TextAttr): pass


class type(xsc.Attrs):
	class type(xsc.TextAttr):
		values = ("identity", "table", "discrete", "linear", "gamma")
		required = True


class preserveAspectRatio(xsc.Attrs):
	class preserveAspectRatio(xsc.TextAttr): pass


class z(xsc.Attrs):
	class z(xsc.TextAttr): pass


class specularExponent(xsc.Attrs):
	class specularExponent(xsc.TextAttr): pass


class vert_adv_y(xsc.Attrs):
	class vert_adv_y(xsc.TextAttr): xmlname = "vert-adv-y"


class vert_origin_x(xsc.Attrs):
	class vert_origin_x(xsc.TextAttr): xmlname = "vert-origin-x"


class vert_origin_y(xsc.Attrs):
	class vert_origin_y(xsc.TextAttr): xmlname = "vert-origin-y"


class height2(xsc.Attrs):
	class height(xsc.TextAttr): required = True


class width2(xsc.Attrs):
	class width(xsc.TextAttr): required = True


class d(xsc.Attrs):
	class d(xsc.TextAttr): pass


class horiz_adv_x(xsc.Attrs):
	class horiz_adv_x(xsc.TextAttr): xmlname = "horiz-adv-x"


class g1(xsc.Attrs):
	class g1(xsc.TextAttr): pass


class g2(xsc.Attrs):
	class g2(xsc.TextAttr): pass


class k(xsc.Attrs):
	class k(xsc.TextAttr): required = True


class u1(xsc.Attrs):
	class u1(xsc.TextAttr): pass


class u2(xsc.Attrs):
	class u2(xsc.TextAttr): pass


class x1(xsc.Attrs):
	class x1(xsc.TextAttr): pass


class x2(xsc.Attrs):
	class x2(xsc.TextAttr): pass


class y1(xsc.Attrs):
	class y1(xsc.TextAttr): pass


class y2(xsc.Attrs):
	class y2(xsc.TextAttr): pass


class gradientTransform(xsc.Attrs):
	class gradientTransform(xsc.TextAttr): pass


class gradientUnits(xsc.Attrs):
	class gradientUnits(xsc.TextAttr): values = ("userSpaceOnUse", "objectBoundingBox")


class spreadMethod(xsc.Attrs):
	class spreadMethod(xsc.TextAttr): values = ("pad", "reflect", "repeat")


class viewBox(xsc.Attrs):
	class viewBox(xsc.TextAttr): pass


class points(xsc.Attrs):
	class points(xsc.TextAttr): required = True


class type2(xsc.Attrs):
	class type(xsc.TextAttr): required = True


class zoomAndPan(xsc.Attrs):
	class zoomAndPan(xsc.TextAttr): values = ("disable", "magnify")


class lengthAdjust(xsc.Attrs):
	class lengthAdjust(xsc.TextAttr): values = ("spacing", "spacingAndGlyphs")


class textLength(xsc.Attrs):
	class textLength(xsc.TextAttr): pass


class a(xsc.Element):
	xmlns = xmlns
	class Attrs(alignment_baseline, baseline_shift, class_, clip, clip_path, clip_rule, color, color_interpolation, color_interpolation_filters, color_profile2, color_rendering, cursor2, direction, display, dominant_baseline, enable_background, externalResourcesRequired, fill, fill_opacity, fill_rule, filter2, flood_color, flood_opacity, font_family, font_size, font_size_adjust, font_stretch, font_style, font_variant, font_weight, glyph_orientation_horizontal, glyph_orientation_vertical, id, image_rendering, kerning, letter_spacing, lighting_color, marker_end, marker_mid, marker_start, mask2, onactivate, onclick, onfocusin, onfocusout, onload, onmousedown, onmousemove, onmouseout, onmouseover, onmouseup, opacity, overflow, pointer_events, requiredExtensions, requiredFeatures, shape_rendering, stop_color, stop_opacity, stroke, stroke_dasharray, stroke_dashoffset, stroke_linecap, stroke_linejoin, stroke_miterlimit, stroke_opacity, stroke_width, style2, systemLanguage, text_anchor, text_decoration, text_rendering, transform, unicode_bidi, visibility, word_spacing, writing_mode):
		class target(xsc.TextAttr): pass


class altGlyph(xsc.Element):
	xmlns = xmlns
	class Attrs(alignment_baseline, baseline_shift, class_, clip_path, clip_rule, color, color_interpolation, color_rendering, cursor2, direction, display, dominant_baseline, dx, dy, externalResourcesRequired, fill, fill_opacity, fill_rule, filter2, font_family, font_size, font_size_adjust, font_stretch, font_style, font_variant, font_weight, format, glyph_orientation_horizontal, glyph_orientation_vertical, glyphRef2, id, image_rendering, kerning, letter_spacing, mask2, onactivate, onclick, onfocusin, onfocusout, onload, onmousedown, onmousemove, onmouseout, onmouseover, onmouseup, opacity, pointer_events, requiredExtensions, requiredFeatures, rotate, shape_rendering, stroke, stroke_dasharray, stroke_dashoffset, stroke_linecap, stroke_linejoin, stroke_miterlimit, stroke_opacity, stroke_width, style2, systemLanguage, text_anchor, text_decoration, text_rendering, unicode_bidi, visibility, word_spacing, x, y):
		pass


class altGlyphDef(xsc.Element):
	xmlns = xmlns
	class Attrs(id):
		pass


class altGlyphItem(xsc.Element):
	xmlns = xmlns
	class Attrs(id):
		pass


class animate(xsc.Element):
	xmlns = xmlns
	class Attrs(accumulate, additive, attributeName, attributeType, begin, by, calcMode, dur, end, externalResourcesRequired, fill2, from_, id, keySplines, keyTimes, max, min, onbegin, onend, onload, onrepeat, repeatCount, repeatDur, requiredExtensions, requiredFeatures, restart, systemLanguage, to, values):
		pass


class animateColor(xsc.Element):
	xmlns = xmlns
	class Attrs(accumulate, additive, attributeName, attributeType, begin, by, calcMode, dur, end, externalResourcesRequired, fill2, from_, id, keySplines, keyTimes, max, min, onbegin, onend, onload, onrepeat, repeatCount, repeatDur, requiredExtensions, requiredFeatures, restart, systemLanguage, to, values):
		pass


class animateMotion(xsc.Element):
	xmlns = xmlns
	class Attrs(accumulate, additive, begin, by, dur, end, externalResourcesRequired, fill2, from_, id, keySplines, keyTimes, max, min, onbegin, onend, onload, onrepeat, repeatCount, repeatDur, requiredExtensions, requiredFeatures, restart, rotate, systemLanguage, to, values):
		class calcMode(xsc.TextAttr): values = ("discrete", "linear", "paced", "spline")
		class keyPoints(xsc.TextAttr): pass
		class origin(xsc.TextAttr): pass
		class path(xsc.TextAttr): pass


class animateTransform(xsc.Element):
	xmlns = xmlns
	class Attrs(accumulate, additive, attributeName, attributeType, begin, by, calcMode, dur, end, externalResourcesRequired, fill2, from_, id, keySplines, keyTimes, max, min, onbegin, onend, onload, onrepeat, repeatCount, repeatDur, requiredExtensions, requiredFeatures, restart, systemLanguage, to, values):
		class type(xsc.TextAttr): values = ("translate", "scale", "rotate", "skewX", "skewY")


class circle(xsc.Element):
	xmlns = xmlns
	class Attrs(class_, clip_path, clip_rule, color, color_interpolation, color_rendering, cursor2, cx, cy, display, externalResourcesRequired, fill, fill_opacity, fill_rule, filter2, id, image_rendering, mask2, onactivate, onclick, onfocusin, onfocusout, onload, onmousedown, onmousemove, onmouseout, onmouseover, onmouseup, opacity, pointer_events, requiredExtensions, requiredFeatures, shape_rendering, stroke, stroke_dasharray, stroke_dashoffset, stroke_linecap, stroke_linejoin, stroke_miterlimit, stroke_opacity, stroke_width, style2, systemLanguage, text_rendering, transform, visibility):
		class r(xsc.TextAttr): required = True


class clipPath(xsc.Element):
	xmlns = xmlns
	class Attrs(alignment_baseline, baseline_shift, class_, clip_path, clip_rule, color, color_interpolation, color_rendering, cursor2, direction, display, dominant_baseline, externalResourcesRequired, fill, fill_opacity, fill_rule, filter2, font_family, font_size, font_size_adjust, font_stretch, font_style, font_variant, font_weight, glyph_orientation_horizontal, glyph_orientation_vertical, id, image_rendering, kerning, letter_spacing, mask2, opacity, pointer_events, requiredExtensions, requiredFeatures, shape_rendering, stroke, stroke_dasharray, stroke_dashoffset, stroke_linecap, stroke_linejoin, stroke_miterlimit, stroke_opacity, stroke_width, style2, systemLanguage, text_anchor, text_decoration, text_rendering, transform, unicode_bidi, visibility, word_spacing, writing_mode):
		class clipPathUnits(xsc.TextAttr): values = ("userSpaceOnUse", "objectBoundingBox")


class color_profile(xsc.Element):
	xmlns = xmlns
	xmlname = "color-profile"
	class Attrs(id):
		class local(xsc.TextAttr): pass
		class name(xsc.TextAttr): required = True
		class rendering_intent(xsc.TextAttr):
			xmlname = "rendering-intent"
			values = ("auto", "perceptual", "relative-colorimetric", "saturation", "absolute-colorimetric")


class cursor(xsc.Element):
	xmlns = xmlns
	class Attrs(externalResourcesRequired, id, requiredExtensions, requiredFeatures, systemLanguage, x, y):
		pass


class definition_src(xsc.Element):
	xmlns = xmlns
	xmlname = "definition-src"
	class Attrs(id):
		pass


class defs(xsc.Element):
	xmlns = xmlns
	class Attrs(alignment_baseline, baseline_shift, class_, clip, clip_path, clip_rule, color, color_interpolation, color_interpolation_filters, color_profile2, color_rendering, cursor2, direction, display, dominant_baseline, enable_background, externalResourcesRequired, fill, fill_opacity, fill_rule, filter2, flood_color, flood_opacity, font_family, font_size, font_size_adjust, font_stretch, font_style, font_variant, font_weight, glyph_orientation_horizontal, glyph_orientation_vertical, id, image_rendering, kerning, letter_spacing, lighting_color, marker_end, marker_mid, marker_start, mask2, onactivate, onclick, onfocusin, onfocusout, onload, onmousedown, onmousemove, onmouseout, onmouseover, onmouseup, opacity, overflow, pointer_events, requiredExtensions, requiredFeatures, shape_rendering, stop_color, stop_opacity, stroke, stroke_dasharray, stroke_dashoffset, stroke_linecap, stroke_linejoin, stroke_miterlimit, stroke_opacity, stroke_width, style2, systemLanguage, text_anchor, text_decoration, text_rendering, transform, unicode_bidi, visibility, word_spacing, writing_mode):
		pass


class desc(xsc.Element):
	xmlns = xmlns
	class Attrs(class_, id, style2):
		pass


class ellipse(xsc.Element):
	xmlns = xmlns
	class Attrs(class_, clip_path, clip_rule, color, color_interpolation, color_rendering, cursor2, cx, cy, display, externalResourcesRequired, fill, fill_opacity, fill_rule, filter2, id, image_rendering, mask2, onactivate, onclick, onfocusin, onfocusout, onload, onmousedown, onmousemove, onmouseout, onmouseover, onmouseup, opacity, pointer_events, requiredExtensions, requiredFeatures, shape_rendering, stroke, stroke_dasharray, stroke_dashoffset, stroke_linecap, stroke_linejoin, stroke_miterlimit, stroke_opacity, stroke_width, style2, systemLanguage, text_rendering, transform, visibility):
		class rx(xsc.TextAttr): required = True
		class ry(xsc.TextAttr): required = True


class feBlend(xsc.Element):
	xmlns = xmlns
	class Attrs(color_interpolation_filters, height, id, in_, in2, result, width, x, y):
		class mode(xsc.TextAttr): values = ("normal", "multiply", "screen", "darken", "lighten")


class feColorMatrix(xsc.Element):
	xmlns = xmlns
	class Attrs(color_interpolation_filters, height, id, in_, result, values, width, x, y):
		class type(xsc.TextAttr): values = ("matrix", "saturate", "hueRotate", "luminanceToAlpha")


class feComponentTransfer(xsc.Element):
	xmlns = xmlns
	class Attrs(color_interpolation_filters, height, id, in_, result, width, x, y):
		pass


class feComposite(xsc.Element):
	xmlns = xmlns
	class Attrs(color_interpolation_filters, height, id, in_, in2, result, width, x, y):
		class k1(xsc.TextAttr): pass
		class k2(xsc.TextAttr): pass
		class k3(xsc.TextAttr): pass
		class k4(xsc.TextAttr): pass
		class operator(xsc.TextAttr): values = ("over", "in", "out", "atop", "xor", "arithmetic")


class feConvolveMatrix(xsc.Element):
	xmlns = xmlns
	class Attrs(color_interpolation_filters, height, id, in_, kernelUnitLength, result, width, x, y):
		class bias(xsc.TextAttr): pass
		class divisor(xsc.TextAttr): pass
		class edgeMode(xsc.TextAttr): values = ("duplicate", "wrap", "none")
		class kernelMatrix(xsc.TextAttr): required = True
		class order(xsc.TextAttr): required = True
		class preserveAlpha(xsc.TextAttr): values = ("false", "true")
		class targetX(xsc.IntAttr): pass
		class targetY(xsc.IntAttr): pass


class feDiffuseLighting(xsc.Element):
	xmlns = xmlns
	class Attrs(class_, color, color_interpolation, color_interpolation_filters, color_rendering, height, id, in_, kernelUnitLength, lighting_color, result, style2, surfaceScale, width, x, y):
		class diffuseConstant(xsc.TextAttr): pass


class feDisplacementMap(xsc.Element):
	xmlns = xmlns
	class Attrs(color_interpolation_filters, height, id, in_, in2, result, width, x, y):
		class scale(xsc.TextAttr): pass
		class xChannelSelector(xsc.TextAttr): values = ("R", "G", "B", "A")
		class yChannelSelector(xsc.TextAttr): values = ("R", "G", "B", "A")


class feDistantLight(xsc.Element):
	xmlns = xmlns
	class Attrs(id):
		class azimuth(xsc.TextAttr): pass
		class elevation(xsc.TextAttr): pass


class feFlood(xsc.Element):
	xmlns = xmlns
	class Attrs(class_, color, color_interpolation, color_interpolation_filters, color_rendering, flood_color, flood_opacity, height, id, in_, result, style2, width, x, y):
		pass


class feFuncA(xsc.Element):
	xmlns = xmlns
	class Attrs(amplitude, exponent, id, intercept, offset, slope, tableValues, type):
		pass


class feFuncB(xsc.Element):
	xmlns = xmlns
	class Attrs(amplitude, exponent, id, intercept, offset, slope, tableValues, type):
		pass


class feFuncG(xsc.Element):
	xmlns = xmlns
	class Attrs(amplitude, exponent, id, intercept, offset, slope, tableValues, type):
		pass


class feFuncR(xsc.Element):
	xmlns = xmlns
	class Attrs(amplitude, exponent, id, intercept, offset, slope, tableValues, type):
		pass


class feGaussianBlur(xsc.Element):
	xmlns = xmlns
	class Attrs(color_interpolation_filters, height, id, in_, result, width, x, y):
		class stdDeviation(xsc.TextAttr): pass


class feImage(xsc.Element):
	xmlns = xmlns
	class Attrs(alignment_baseline, baseline_shift, class_, clip, clip_path, clip_rule, color, color_interpolation, color_interpolation_filters, color_profile2, color_rendering, cursor2, direction, display, dominant_baseline, enable_background, externalResourcesRequired, fill, fill_opacity, fill_rule, filter2, flood_color, flood_opacity, font_family, font_size, font_size_adjust, font_stretch, font_style, font_variant, font_weight, glyph_orientation_horizontal, glyph_orientation_vertical, height, id, image_rendering, kerning, letter_spacing, lighting_color, marker_end, marker_mid, marker_start, mask2, opacity, overflow, pointer_events, preserveAspectRatio, result, shape_rendering, stop_color, stop_opacity, stroke, stroke_dasharray, stroke_dashoffset, stroke_linecap, stroke_linejoin, stroke_miterlimit, stroke_opacity, stroke_width, style2, text_anchor, text_decoration, text_rendering, unicode_bidi, visibility, width, word_spacing, writing_mode, x, y):
		pass


class feMerge(xsc.Element):
	xmlns = xmlns
	class Attrs(color_interpolation_filters, height, id, result, width, x, y):
		pass


class feMergeNode(xsc.Element):
	xmlns = xmlns
	class Attrs(id, in_):
		pass


class feMorphology(xsc.Element):
	xmlns = xmlns
	class Attrs(color_interpolation_filters, height, id, in_, result, width, x, y):
		class operator(xsc.TextAttr): values = ("erode", "dilate")
		class radius(xsc.TextAttr): pass


class feOffset(xsc.Element):
	xmlns = xmlns
	class Attrs(color_interpolation_filters, dx, dy, height, id, in_, result, width, x, y):
		pass


class fePointLight(xsc.Element):
	xmlns = xmlns
	class Attrs(id, x, y, z):
		pass


class feSpecularLighting(xsc.Element):
	xmlns = xmlns
	class Attrs(class_, color, color_interpolation, color_interpolation_filters, color_rendering, height, id, in_, kernelUnitLength, lighting_color, result, specularExponent, style2, surfaceScale, width, x, y):
		class specularConstant(xsc.TextAttr): pass


class feSpotLight(xsc.Element):
	xmlns = xmlns
	class Attrs(id, specularExponent, x, y, z):
		class limitingConeAngle(xsc.TextAttr): pass
		class pointsAtX(xsc.TextAttr): pass
		class pointsAtY(xsc.TextAttr): pass
		class pointsAtZ(xsc.TextAttr): pass


class feTile(xsc.Element):
	xmlns = xmlns
	class Attrs(color_interpolation_filters, height, id, in_, result, width, x, y):
		pass


class feTurbulence(xsc.Element):
	xmlns = xmlns
	class Attrs(color_interpolation_filters, height, id, result, width, x, y):
		class baseFrequency(xsc.TextAttr): pass
		class numOctaves(xsc.IntAttr): pass
		class seed(xsc.TextAttr): pass
		class stitchTiles(xsc.TextAttr): values = ("stitch", "noStitch")
		class type(xsc.TextAttr): values = ("fractalNoise", "turbulence")


class filter(xsc.Element):
	xmlns = xmlns
	class Attrs(alignment_baseline, baseline_shift, class_, clip, clip_path, clip_rule, color, color_interpolation, color_interpolation_filters, color_profile2, color_rendering, cursor2, direction, display, dominant_baseline, enable_background, externalResourcesRequired, fill, fill_opacity, fill_rule, filter2, flood_color, flood_opacity, font_family, font_size, font_size_adjust, font_stretch, font_style, font_variant, font_weight, glyph_orientation_horizontal, glyph_orientation_vertical, height, id, image_rendering, kerning, letter_spacing, lighting_color, marker_end, marker_mid, marker_start, mask2, opacity, overflow, pointer_events, shape_rendering, stop_color, stop_opacity, stroke, stroke_dasharray, stroke_dashoffset, stroke_linecap, stroke_linejoin, stroke_miterlimit, stroke_opacity, stroke_width, style2, text_anchor, text_decoration, text_rendering, unicode_bidi, visibility, width, word_spacing, writing_mode, x, y):
		class filterRes(xsc.TextAttr): pass
		class filterUnits(xsc.TextAttr): values = ("userSpaceOnUse", "objectBoundingBox")
		class primitiveUnits(xsc.TextAttr): values = ("userSpaceOnUse", "objectBoundingBox")


class font(xsc.Element):
	xmlns = xmlns
	class Attrs(alignment_baseline, baseline_shift, class_, clip, clip_path, clip_rule, color, color_interpolation, color_interpolation_filters, color_profile2, color_rendering, cursor2, direction, display, dominant_baseline, enable_background, externalResourcesRequired, fill, fill_opacity, fill_rule, filter2, flood_color, flood_opacity, font_family, font_size, font_size_adjust, font_stretch, font_style, font_variant, font_weight, glyph_orientation_horizontal, glyph_orientation_vertical, id, image_rendering, kerning, letter_spacing, lighting_color, marker_end, marker_mid, marker_start, mask2, opacity, overflow, pointer_events, shape_rendering, stop_color, stop_opacity, stroke, stroke_dasharray, stroke_dashoffset, stroke_linecap, stroke_linejoin, stroke_miterlimit, stroke_opacity, stroke_width, style2, text_anchor, text_decoration, text_rendering, unicode_bidi, vert_adv_y, vert_origin_x, vert_origin_y, visibility, word_spacing, writing_mode):
		class horiz_adv_x(xsc.TextAttr):
			xmlname = "horiz-adv-x"
			required = True
		class horiz_origin_x(xsc.TextAttr): xmlname = "horiz-origin-x"
		class horiz_origin_y(xsc.TextAttr): xmlname = "horiz-origin-y"


class font_face(xsc.Element):
	xmlns = xmlns
	xmlname = "font-face"
	class Attrs(font_family, font_size, id, slope):
		class accent_height(xsc.TextAttr): xmlname = "accent-height"
		class alphabetic(xsc.TextAttr): pass
		class ascent(xsc.TextAttr): pass
		class bbox(xsc.TextAttr): pass
		class cap_height(xsc.TextAttr): xmlname = "cap-height"
		class descent(xsc.TextAttr): pass
		class font_stretch(xsc.TextAttr): xmlname = "font-stretch"
		class font_style(xsc.TextAttr): xmlname = "font-style"
		class font_variant(xsc.TextAttr): xmlname = "font-variant"
		class font_weight(xsc.TextAttr): xmlname = "font-weight"
		class hanging(xsc.TextAttr): pass
		class ideographic(xsc.TextAttr): pass
		class mathematical(xsc.TextAttr): pass
		class overline_position(xsc.TextAttr): xmlname = "overline-position"
		class overline_thickness(xsc.TextAttr): xmlname = "overline-thickness"
		class panose_1(xsc.TextAttr): xmlname = "panose-1"
		class stemh(xsc.TextAttr): pass
		class stemv(xsc.TextAttr): pass
		class strikethrough_position(xsc.TextAttr): xmlname = "strikethrough-position"
		class strikethrough_thickness(xsc.TextAttr): xmlname = "strikethrough-thickness"
		class underline_position(xsc.TextAttr): xmlname = "underline-position"
		class underline_thickness(xsc.TextAttr): xmlname = "underline-thickness"
		class unicode_range(xsc.TextAttr): xmlname = "unicode-range"
		class units_per_em(xsc.TextAttr): xmlname = "units-per-em"
		class v_alphabetic(xsc.TextAttr): xmlname = "v-alphabetic"
		class v_hanging(xsc.TextAttr): xmlname = "v-hanging"
		class v_ideographic(xsc.TextAttr): xmlname = "v-ideographic"
		class v_mathematical(xsc.TextAttr): xmlname = "v-mathematical"
		class widths(xsc.TextAttr): pass
		class x_height(xsc.TextAttr): xmlname = "x-height"


class font_face_format(xsc.Element):
	xmlns = xmlns
	xmlname = "font-face-format"
	class Attrs(id):
		class string(xsc.TextAttr): pass


class font_face_name(xsc.Element):
	xmlns = xmlns
	xmlname = "font-face-name"
	class Attrs(id):
		class name(xsc.TextAttr): pass


class font_face_src(xsc.Element):
	xmlns = xmlns
	xmlname = "font-face-src"
	class Attrs(id):
		pass


class font_face_uri(xsc.Element):
	xmlns = xmlns
	xmlname = "font-face-uri"
	class Attrs(id):
		pass


class foreignObject(xsc.Element):
	xmlns = xmlns
	class Attrs(alignment_baseline, baseline_shift, class_, clip, clip_path, clip_rule, color, color_interpolation, color_interpolation_filters, color_profile2, color_rendering, cursor2, direction, display, dominant_baseline, enable_background, externalResourcesRequired, fill, fill_opacity, fill_rule, filter2, flood_color, flood_opacity, font_family, font_size, font_size_adjust, font_stretch, font_style, font_variant, font_weight, glyph_orientation_horizontal, glyph_orientation_vertical, height2, id, image_rendering, kerning, letter_spacing, lighting_color, marker_end, marker_mid, marker_start, mask2, onactivate, onclick, onfocusin, onfocusout, onload, onmousedown, onmousemove, onmouseout, onmouseover, onmouseup, opacity, overflow, pointer_events, requiredExtensions, requiredFeatures, shape_rendering, stop_color, stop_opacity, stroke, stroke_dasharray, stroke_dashoffset, stroke_linecap, stroke_linejoin, stroke_miterlimit, stroke_opacity, stroke_width, style2, systemLanguage, text_anchor, text_decoration, text_rendering, transform, unicode_bidi, visibility, width2, word_spacing, writing_mode, x, y):
		pass


class g(xsc.Element):
	xmlns = xmlns
	class Attrs(alignment_baseline, baseline_shift, class_, clip, clip_path, clip_rule, color, color_interpolation, color_interpolation_filters, color_profile2, color_rendering, cursor2, direction, display, dominant_baseline, enable_background, externalResourcesRequired, fill, fill_opacity, fill_rule, filter2, flood_color, flood_opacity, font_family, font_size, font_size_adjust, font_stretch, font_style, font_variant, font_weight, glyph_orientation_horizontal, glyph_orientation_vertical, id, image_rendering, kerning, letter_spacing, lighting_color, marker_end, marker_mid, marker_start, mask2, onactivate, onclick, onfocusin, onfocusout, onload, onmousedown, onmousemove, onmouseout, onmouseover, onmouseup, opacity, overflow, pointer_events, requiredExtensions, requiredFeatures, shape_rendering, stop_color, stop_opacity, stroke, stroke_dasharray, stroke_dashoffset, stroke_linecap, stroke_linejoin, stroke_miterlimit, stroke_opacity, stroke_width, style2, systemLanguage, text_anchor, text_decoration, text_rendering, transform, unicode_bidi, visibility, word_spacing, writing_mode):
		pass


class glyph(xsc.Element):
	xmlns = xmlns
	class Attrs(alignment_baseline, baseline_shift, class_, clip, clip_path, clip_rule, color, color_interpolation, color_interpolation_filters, color_profile2, color_rendering, cursor2, d, direction, display, dominant_baseline, enable_background, fill, fill_opacity, fill_rule, filter2, flood_color, flood_opacity, font_family, font_size, font_size_adjust, font_stretch, font_style, font_variant, font_weight, glyph_orientation_horizontal, glyph_orientation_vertical, horiz_adv_x, id, image_rendering, kerning, letter_spacing, lighting_color, marker_end, marker_mid, marker_start, mask2, opacity, overflow, pointer_events, shape_rendering, stop_color, stop_opacity, stroke, stroke_dasharray, stroke_dashoffset, stroke_linecap, stroke_linejoin, stroke_miterlimit, stroke_opacity, stroke_width, style2, text_anchor, text_decoration, text_rendering, unicode_bidi, vert_adv_y, vert_origin_x, vert_origin_y, visibility, word_spacing, writing_mode):
		class arabic_form(xsc.TextAttr): xmlname = "arabic-form"
		class glyph_name(xsc.TextAttr): xmlname = "glyph-name"
		class lang(xsc.TextAttr): pass
		class orientation(xsc.TextAttr): pass
		class str(xsc.TextAttr): pass


class glyphRef(xsc.Element):
	xmlns = xmlns
	class Attrs(class_, dx, dy, font_family, font_size, font_size_adjust, font_stretch, font_style, font_variant, font_weight, format, glyphRef2, id, style2, x, y):
		pass


class hkern(xsc.Element):
	xmlns = xmlns
	class Attrs(g1, g2, id, k, u1, u2):
		pass


class image(xsc.Element):
	xmlns = xmlns
	class Attrs(class_, clip, clip_path, clip_rule, color, color_interpolation, color_profile2, color_rendering, cursor2, display, externalResourcesRequired, fill_opacity, filter2, height2, id, image_rendering, mask2, onactivate, onclick, onfocusin, onfocusout, onload, onmousedown, onmousemove, onmouseout, onmouseover, onmouseup, opacity, overflow, pointer_events, preserveAspectRatio, requiredExtensions, requiredFeatures, shape_rendering, stroke_opacity, style2, systemLanguage, text_rendering, transform, visibility, width2, x, y):
		pass


class line(xsc.Element):
	xmlns = xmlns
	class Attrs(class_, clip_path, clip_rule, color, color_interpolation, color_rendering, cursor2, display, externalResourcesRequired, fill, fill_opacity, fill_rule, filter2, id, image_rendering, marker_end, marker_mid, marker_start, mask2, onactivate, onclick, onfocusin, onfocusout, onload, onmousedown, onmousemove, onmouseout, onmouseover, onmouseup, opacity, pointer_events, requiredExtensions, requiredFeatures, shape_rendering, stroke, stroke_dasharray, stroke_dashoffset, stroke_linecap, stroke_linejoin, stroke_miterlimit, stroke_opacity, stroke_width, style2, systemLanguage, text_rendering, transform, visibility, x1, x2, y1, y2):
		pass


class linearGradient(xsc.Element):
	xmlns = xmlns
	class Attrs(class_, color, color_interpolation, color_rendering, externalResourcesRequired, gradientTransform, gradientUnits, id, spreadMethod, stop_color, stop_opacity, style2, x1, x2, y1, y2):
		pass


class marker(xsc.Element):
	xmlns = xmlns
	class Attrs(alignment_baseline, baseline_shift, class_, clip, clip_path, clip_rule, color, color_interpolation, color_interpolation_filters, color_profile2, color_rendering, cursor2, direction, display, dominant_baseline, enable_background, externalResourcesRequired, fill, fill_opacity, fill_rule, filter2, flood_color, flood_opacity, font_family, font_size, font_size_adjust, font_stretch, font_style, font_variant, font_weight, glyph_orientation_horizontal, glyph_orientation_vertical, id, image_rendering, kerning, letter_spacing, lighting_color, marker_end, marker_mid, marker_start, mask2, opacity, overflow, pointer_events, preserveAspectRatio, shape_rendering, stop_color, stop_opacity, stroke, stroke_dasharray, stroke_dashoffset, stroke_linecap, stroke_linejoin, stroke_miterlimit, stroke_opacity, stroke_width, style2, text_anchor, text_decoration, text_rendering, unicode_bidi, viewBox, visibility, word_spacing, writing_mode):
		class markerHeight(xsc.TextAttr): pass
		class markerUnits(xsc.TextAttr): values = ("strokeWidth", "userSpaceOnUse")
		class markerWidth(xsc.TextAttr): pass
		class orient(xsc.TextAttr): pass
		class refX(xsc.TextAttr): pass
		class refY(xsc.TextAttr): pass


class mask(xsc.Element):
	xmlns = xmlns
	class Attrs(alignment_baseline, baseline_shift, class_, clip, clip_path, clip_rule, color, color_interpolation, color_interpolation_filters, color_profile2, color_rendering, cursor2, direction, display, dominant_baseline, enable_background, externalResourcesRequired, fill, fill_opacity, fill_rule, filter2, flood_color, flood_opacity, font_family, font_size, font_size_adjust, font_stretch, font_style, font_variant, font_weight, glyph_orientation_horizontal, glyph_orientation_vertical, height, id, image_rendering, kerning, letter_spacing, lighting_color, marker_end, marker_mid, marker_start, mask2, opacity, overflow, pointer_events, requiredExtensions, requiredFeatures, shape_rendering, stop_color, stop_opacity, stroke, stroke_dasharray, stroke_dashoffset, stroke_linecap, stroke_linejoin, stroke_miterlimit, stroke_opacity, stroke_width, style2, systemLanguage, text_anchor, text_decoration, text_rendering, unicode_bidi, visibility, width, word_spacing, writing_mode, x, y):
		class maskContentUnits(xsc.TextAttr): values = ("userSpaceOnUse", "objectBoundingBox")
		class maskUnits(xsc.TextAttr): values = ("userSpaceOnUse", "objectBoundingBox")


class metadata(xsc.Element):
	xmlns = xmlns
	class Attrs(id):
		pass


class missing_glyph(xsc.Element):
	xmlns = xmlns
	xmlname = "missing-glyph"
	class Attrs(alignment_baseline, baseline_shift, class_, clip, clip_path, clip_rule, color, color_interpolation, color_interpolation_filters, color_profile2, color_rendering, cursor2, d, direction, display, dominant_baseline, enable_background, fill, fill_opacity, fill_rule, filter2, flood_color, flood_opacity, font_family, font_size, font_size_adjust, font_stretch, font_style, font_variant, font_weight, glyph_orientation_horizontal, glyph_orientation_vertical, horiz_adv_x, id, image_rendering, kerning, letter_spacing, lighting_color, marker_end, marker_mid, marker_start, mask2, opacity, overflow, pointer_events, shape_rendering, stop_color, stop_opacity, stroke, stroke_dasharray, stroke_dashoffset, stroke_linecap, stroke_linejoin, stroke_miterlimit, stroke_opacity, stroke_width, style2, text_anchor, text_decoration, text_rendering, unicode_bidi, vert_adv_y, vert_origin_x, vert_origin_y, visibility, word_spacing, writing_mode):
		pass


class mpath(xsc.Element):
	xmlns = xmlns
	class Attrs(externalResourcesRequired, id):
		pass


class path(xsc.Element):
	xmlns = xmlns
	class Attrs(class_, clip_path, clip_rule, color, color_interpolation, color_rendering, cursor2, display, externalResourcesRequired, fill, fill_opacity, fill_rule, filter2, id, image_rendering, marker_end, marker_mid, marker_start, mask2, onactivate, onclick, onfocusin, onfocusout, onload, onmousedown, onmousemove, onmouseout, onmouseover, onmouseup, opacity, pointer_events, requiredExtensions, requiredFeatures, shape_rendering, stroke, stroke_dasharray, stroke_dashoffset, stroke_linecap, stroke_linejoin, stroke_miterlimit, stroke_opacity, stroke_width, style2, systemLanguage, text_rendering, transform, visibility):
		class d(xsc.TextAttr): required = True
		class pathLength(xsc.TextAttr): pass


class pattern(xsc.Element):
	xmlns = xmlns
	class Attrs(alignment_baseline, baseline_shift, class_, clip, clip_path, clip_rule, color, color_interpolation, color_interpolation_filters, color_profile2, color_rendering, cursor2, direction, display, dominant_baseline, enable_background, externalResourcesRequired, fill, fill_opacity, fill_rule, filter2, flood_color, flood_opacity, font_family, font_size, font_size_adjust, font_stretch, font_style, font_variant, font_weight, glyph_orientation_horizontal, glyph_orientation_vertical, height, id, image_rendering, kerning, letter_spacing, lighting_color, marker_end, marker_mid, marker_start, mask2, opacity, overflow, pointer_events, preserveAspectRatio, requiredExtensions, requiredFeatures, shape_rendering, stop_color, stop_opacity, stroke, stroke_dasharray, stroke_dashoffset, stroke_linecap, stroke_linejoin, stroke_miterlimit, stroke_opacity, stroke_width, style2, systemLanguage, text_anchor, text_decoration, text_rendering, unicode_bidi, viewBox, visibility, width, word_spacing, writing_mode, x, y):
		class patternContentUnits(xsc.TextAttr): values = ("userSpaceOnUse", "objectBoundingBox")
		class patternTransform(xsc.TextAttr): pass
		class patternUnits(xsc.TextAttr): values = ("userSpaceOnUse", "objectBoundingBox")


class polygon(xsc.Element):
	xmlns = xmlns
	class Attrs(class_, clip_path, clip_rule, color, color_interpolation, color_rendering, cursor2, display, externalResourcesRequired, fill, fill_opacity, fill_rule, filter2, id, image_rendering, marker_end, marker_mid, marker_start, mask2, onactivate, onclick, onfocusin, onfocusout, onload, onmousedown, onmousemove, onmouseout, onmouseover, onmouseup, opacity, pointer_events, points, requiredExtensions, requiredFeatures, shape_rendering, stroke, stroke_dasharray, stroke_dashoffset, stroke_linecap, stroke_linejoin, stroke_miterlimit, stroke_opacity, stroke_width, style2, systemLanguage, text_rendering, transform, visibility):
		pass


class polyline(xsc.Element):
	xmlns = xmlns
	class Attrs(class_, clip_path, clip_rule, color, color_interpolation, color_rendering, cursor2, display, externalResourcesRequired, fill, fill_opacity, fill_rule, filter2, id, image_rendering, marker_end, marker_mid, marker_start, mask2, onactivate, onclick, onfocusin, onfocusout, onload, onmousedown, onmousemove, onmouseout, onmouseover, onmouseup, opacity, pointer_events, points, requiredExtensions, requiredFeatures, shape_rendering, stroke, stroke_dasharray, stroke_dashoffset, stroke_linecap, stroke_linejoin, stroke_miterlimit, stroke_opacity, stroke_width, style2, systemLanguage, text_rendering, transform, visibility):
		pass


class radialGradient(xsc.Element):
	xmlns = xmlns
	class Attrs(class_, color, color_interpolation, color_rendering, cx, cy, externalResourcesRequired, gradientTransform, gradientUnits, id, spreadMethod, stop_color, stop_opacity, style2):
		class fx(xsc.TextAttr): pass
		class fy(xsc.TextAttr): pass
		class r(xsc.TextAttr): pass


class rect(xsc.Element):
	xmlns = xmlns
	class Attrs(class_, clip_path, clip_rule, color, color_interpolation, color_rendering, cursor2, display, externalResourcesRequired, fill, fill_opacity, fill_rule, filter2, height2, id, image_rendering, mask2, onactivate, onclick, onfocusin, onfocusout, onload, onmousedown, onmousemove, onmouseout, onmouseover, onmouseup, opacity, pointer_events, requiredExtensions, requiredFeatures, shape_rendering, stroke, stroke_dasharray, stroke_dashoffset, stroke_linecap, stroke_linejoin, stroke_miterlimit, stroke_opacity, stroke_width, style2, systemLanguage, text_rendering, transform, visibility, width2, x, y):
		class rx(xsc.TextAttr): pass
		class ry(xsc.TextAttr): pass


class script(xsc.Element):
	xmlns = xmlns
	class Attrs(externalResourcesRequired, id, type2):
		pass


class set(xsc.Element):
	xmlns = xmlns
	class Attrs(attributeName, attributeType, begin, dur, end, externalResourcesRequired, fill2, id, max, min, onbegin, onend, onload, onrepeat, repeatCount, repeatDur, requiredExtensions, requiredFeatures, restart, systemLanguage, to):
		pass


class stop(xsc.Element):
	xmlns = xmlns
	class Attrs(class_, color, color_interpolation, color_rendering, id, stop_color, stop_opacity, style2):
		class offset(xsc.TextAttr): required = True


class style(xsc.Element):
	xmlns = xmlns
	class Attrs(id, type2):
		class media(xsc.TextAttr): pass
		class title(xsc.TextAttr): pass


class svg(xsc.Element):
	xmlns = xmlns
	class Attrs(alignment_baseline, baseline_shift, class_, clip, clip_path, clip_rule, color, color_interpolation, color_interpolation_filters, color_profile2, color_rendering, cursor2, direction, display, dominant_baseline, enable_background, externalResourcesRequired, fill, fill_opacity, fill_rule, filter2, flood_color, flood_opacity, font_family, font_size, font_size_adjust, font_stretch, font_style, font_variant, font_weight, glyph_orientation_horizontal, glyph_orientation_vertical, height, id, image_rendering, kerning, letter_spacing, lighting_color, marker_end, marker_mid, marker_start, mask2, onactivate, onclick, onfocusin, onfocusout, onload, onmousedown, onmousemove, onmouseout, onmouseover, onmouseup, opacity, overflow, pointer_events, preserveAspectRatio, requiredExtensions, requiredFeatures, shape_rendering, stop_color, stop_opacity, stroke, stroke_dasharray, stroke_dashoffset, stroke_linecap, stroke_linejoin, stroke_miterlimit, stroke_opacity, stroke_width, style2, systemLanguage, text_anchor, text_decoration, text_rendering, unicode_bidi, viewBox, visibility, width, word_spacing, writing_mode, x, y, zoomAndPan):
		class baseProfile(xsc.TextAttr): pass
		class contentScriptType(xsc.TextAttr): pass
		class contentStyleType(xsc.TextAttr): pass
		class onabort(xsc.TextAttr): pass
		class onerror(xsc.TextAttr): pass
		class onresize(xsc.TextAttr): pass
		class onscroll(xsc.TextAttr): pass
		class onunload(xsc.TextAttr): pass
		class onzoom(xsc.TextAttr): pass
		class version(xsc.TextAttr): pass


class switch(xsc.Element):
	xmlns = xmlns
	class Attrs(alignment_baseline, baseline_shift, class_, clip, clip_path, clip_rule, color, color_interpolation, color_interpolation_filters, color_profile2, color_rendering, cursor2, direction, display, dominant_baseline, enable_background, externalResourcesRequired, fill, fill_opacity, fill_rule, filter2, flood_color, flood_opacity, font_family, font_size, font_size_adjust, font_stretch, font_style, font_variant, font_weight, glyph_orientation_horizontal, glyph_orientation_vertical, id, image_rendering, kerning, letter_spacing, lighting_color, marker_end, marker_mid, marker_start, mask2, onactivate, onclick, onfocusin, onfocusout, onload, onmousedown, onmousemove, onmouseout, onmouseover, onmouseup, opacity, overflow, pointer_events, requiredExtensions, requiredFeatures, shape_rendering, stop_color, stop_opacity, stroke, stroke_dasharray, stroke_dashoffset, stroke_linecap, stroke_linejoin, stroke_miterlimit, stroke_opacity, stroke_width, style2, systemLanguage, text_anchor, text_decoration, text_rendering, transform, unicode_bidi, visibility, word_spacing, writing_mode):
		pass


class symbol(xsc.Element):
	xmlns = xmlns
	class Attrs(alignment_baseline, baseline_shift, class_, clip, clip_path, clip_rule, color, color_interpolation, color_interpolation_filters, color_profile2, color_rendering, cursor2, direction, display, dominant_baseline, enable_background, externalResourcesRequired, fill, fill_opacity, fill_rule, filter2, flood_color, flood_opacity, font_family, font_size, font_size_adjust, font_stretch, font_style, font_variant, font_weight, glyph_orientation_horizontal, glyph_orientation_vertical, id, image_rendering, kerning, letter_spacing, lighting_color, marker_end, marker_mid, marker_start, mask2, onactivate, onclick, onfocusin, onfocusout, onload, onmousedown, onmousemove, onmouseout, onmouseover, onmouseup, opacity, overflow, pointer_events, preserveAspectRatio, shape_rendering, stop_color, stop_opacity, stroke, stroke_dasharray, stroke_dashoffset, stroke_linecap, stroke_linejoin, stroke_miterlimit, stroke_opacity, stroke_width, style2, text_anchor, text_decoration, text_rendering, unicode_bidi, viewBox, visibility, word_spacing, writing_mode):
		pass


class text(xsc.Element):
	xmlns = xmlns
	class Attrs(alignment_baseline, baseline_shift, class_, clip_path, clip_rule, color, color_interpolation, color_rendering, cursor2, direction, display, dominant_baseline, dx, dy, externalResourcesRequired, fill, fill_opacity, fill_rule, filter2, font_family, font_size, font_size_adjust, font_stretch, font_style, font_variant, font_weight, glyph_orientation_horizontal, glyph_orientation_vertical, id, image_rendering, kerning, lengthAdjust, letter_spacing, mask2, onactivate, onclick, onfocusin, onfocusout, onload, onmousedown, onmousemove, onmouseout, onmouseover, onmouseup, opacity, pointer_events, requiredExtensions, requiredFeatures, rotate, shape_rendering, stroke, stroke_dasharray, stroke_dashoffset, stroke_linecap, stroke_linejoin, stroke_miterlimit, stroke_opacity, stroke_width, style2, systemLanguage, text_anchor, text_decoration, text_rendering, textLength, transform, unicode_bidi, visibility, word_spacing, writing_mode, x, y):
		pass


class textPath(xsc.Element):
	xmlns = xmlns
	class Attrs(alignment_baseline, baseline_shift, class_, clip_path, clip_rule, color, color_interpolation, color_rendering, cursor2, direction, display, dominant_baseline, externalResourcesRequired, fill, fill_opacity, fill_rule, filter2, font_family, font_size, font_size_adjust, font_stretch, font_style, font_variant, font_weight, glyph_orientation_horizontal, glyph_orientation_vertical, id, image_rendering, kerning, lengthAdjust, letter_spacing, mask2, onactivate, onclick, onfocusin, onfocusout, onload, onmousedown, onmousemove, onmouseout, onmouseover, onmouseup, opacity, pointer_events, requiredExtensions, requiredFeatures, shape_rendering, stroke, stroke_dasharray, stroke_dashoffset, stroke_linecap, stroke_linejoin, stroke_miterlimit, stroke_opacity, stroke_width, style2, systemLanguage, text_anchor, text_decoration, text_rendering, textLength, unicode_bidi, visibility, word_spacing):
		class method(xsc.TextAttr): values = ("align", "stretch")
		class spacing(xsc.TextAttr): values = ("auto", "exact")
		class startOffset(xsc.TextAttr): pass


class title(xsc.Element):
	xmlns = xmlns
	class Attrs(class_, id, style2):
		pass


class tref(xsc.Element):
	xmlns = xmlns
	class Attrs(alignment_baseline, baseline_shift, class_, clip_path, clip_rule, color, color_interpolation, color_rendering, cursor2, direction, display, dominant_baseline, dx, dy, externalResourcesRequired, fill, fill_opacity, fill_rule, filter2, font_family, font_size, font_size_adjust, font_stretch, font_style, font_variant, font_weight, glyph_orientation_horizontal, glyph_orientation_vertical, id, image_rendering, kerning, lengthAdjust, letter_spacing, mask2, onactivate, onclick, onfocusin, onfocusout, onload, onmousedown, onmousemove, onmouseout, onmouseover, onmouseup, opacity, pointer_events, requiredExtensions, requiredFeatures, rotate, shape_rendering, stroke, stroke_dasharray, stroke_dashoffset, stroke_linecap, stroke_linejoin, stroke_miterlimit, stroke_opacity, stroke_width, style2, systemLanguage, text_anchor, text_decoration, text_rendering, textLength, unicode_bidi, visibility, word_spacing, x, y):
		pass


class tspan(xsc.Element):
	xmlns = xmlns
	class Attrs(alignment_baseline, baseline_shift, class_, clip_path, clip_rule, color, color_interpolation, color_rendering, cursor2, direction, display, dominant_baseline, dx, dy, externalResourcesRequired, fill, fill_opacity, fill_rule, filter2, font_family, font_size, font_size_adjust, font_stretch, font_style, font_variant, font_weight, glyph_orientation_horizontal, glyph_orientation_vertical, id, image_rendering, kerning, lengthAdjust, letter_spacing, mask2, onactivate, onclick, onfocusin, onfocusout, onload, onmousedown, onmousemove, onmouseout, onmouseover, onmouseup, opacity, pointer_events, requiredExtensions, requiredFeatures, rotate, shape_rendering, stroke, stroke_dasharray, stroke_dashoffset, stroke_linecap, stroke_linejoin, stroke_miterlimit, stroke_opacity, stroke_width, style2, systemLanguage, text_anchor, text_decoration, text_rendering, textLength, unicode_bidi, visibility, word_spacing, x, y):
		pass


class use(xsc.Element):
	xmlns = xmlns
	class Attrs(alignment_baseline, baseline_shift, class_, clip, clip_path, clip_rule, color, color_interpolation, color_interpolation_filters, color_profile2, color_rendering, cursor2, direction, display, dominant_baseline, enable_background, externalResourcesRequired, fill, fill_opacity, fill_rule, filter2, flood_color, flood_opacity, font_family, font_size, font_size_adjust, font_stretch, font_style, font_variant, font_weight, glyph_orientation_horizontal, glyph_orientation_vertical, height, id, image_rendering, kerning, letter_spacing, lighting_color, marker_end, marker_mid, marker_start, mask2, onactivate, onclick, onfocusin, onfocusout, onload, onmousedown, onmousemove, onmouseout, onmouseover, onmouseup, opacity, overflow, pointer_events, requiredExtensions, requiredFeatures, shape_rendering, stop_color, stop_opacity, stroke, stroke_dasharray, stroke_dashoffset, stroke_linecap, stroke_linejoin, stroke_miterlimit, stroke_opacity, stroke_width, style2, systemLanguage, text_anchor, text_decoration, text_rendering, transform, unicode_bidi, visibility, width, word_spacing, writing_mode, x, y):
		pass


class view(xsc.Element):
	xmlns = xmlns
	class Attrs(externalResourcesRequired, id, preserveAspectRatio, viewBox, zoomAndPan):
		class viewTarget(xsc.TextAttr): pass


class vkern(xsc.Element):
	xmlns = xmlns
	class Attrs(g1, g2, id, k, u1, u2):
		pass


altGlyphDef.model = sims.Elements(altGlyphItem, glyphRef)
feBlend.model = \
feColorMatrix.model = \
feComposite.model = \
feConvolveMatrix.model = \
feDisplacementMap.model = \
feDistantLight.model = \
feFuncA.model = \
feFuncB.model = \
feFuncG.model = \
feFuncR.model = \
feGaussianBlur.model = \
feMergeNode.model = \
feMorphology.model = \
feOffset.model = \
fePointLight.model = \
feSpotLight.model = \
feTile.model = \
feTurbulence.model = sims.Elements(animate, set)
feFlood.model = \
stop.model = sims.Elements(animate, set, animateColor)
feImage.model = sims.Elements(animate, set, animateTransform)
circle.model = \
ellipse.model = \
image.model = \
line.model = \
path.model = \
polygon.model = \
polyline.model = \
rect.model = \
use.model = sims.Elements(animateMotion, set, title, animateColor, animateTransform, animate, metadata, desc)
font_face.model = sims.Elements(definition_src, desc, metadata, font_face_src, title)
defs.model = sims.Elements(defs, set, animate, text, symbol, clipPath, use, animateColor, font_face, style, polyline, view, marker, path, line, font, color_profile, ellipse, rect, desc, a, cursor, animateMotion, polygon, g, title, svg, script, mask, altGlyphDef, filter, switch, animateTransform, linearGradient, pattern, circle, radialGradient, image, metadata)
g.model = \
glyph.model = \
marker.model = \
mask.model = \
missing_glyph.model = \
pattern.model = \
svg.model = \
symbol.model = sims.Elements(defs, set, animate, text, symbol, clipPath, use, animateColor, font_face, style, polyline, view, marker, path, line, font, color_profile, ellipse, rect, desc, a, cursor, animateMotion, polygon, g, title, svg, script, mask, altGlyphDef, filter, switch, animateTransform, linearGradient, pattern, circle, radialGradient, image, metadata)
animateMotion.model = sims.Elements(desc, metadata, mpath, title)
feComponentTransfer.model = sims.Elements(feFuncA, feFuncR, feFuncB, feFuncG)
feMerge.model = sims.Elements(feMergeNode)
filter.model = sims.Elements(feTurbulence, set, feTile, feColorMatrix, feConvolveMatrix, feMorphology, feGaussianBlur, feDisplacementMap, feComposite, animate, feMerge, feSpecularLighting, desc, feComponentTransfer, title, feDiffuseLighting, feFlood, feBlend, feOffset, feImage, metadata)
font_face_uri.model = sims.Elements(font_face_format)
font_face_src.model = sims.Elements(font_face_name, font_face_uri)
altGlyphItem.model = sims.Elements(glyphRef)
switch.model = sims.Elements(set, animate, text, image, animateMotion, animateColor, polyline, path, line, ellipse, rect, desc, a, use, polygon, g, title, svg, switch, animateTransform, foreignObject, circle, metadata)
clipPath.model = sims.Elements(set, animate, text, use, animateColor, polyline, path, line, ellipse, rect, desc, animateMotion, polygon, title, altGlyphDef, animateTransform, circle, metadata)
feDiffuseLighting.model = \
feSpecularLighting.model = sims.Elements(set, fePointLight, feSpotLight, animate, feDistantLight, animateColor)
tref.model = sims.Elements(set, title, animate, metadata, animateColor, desc)
linearGradient.model = \
radialGradient.model = sims.Elements(set, title, animate, metadata, stop, animateTransform, desc)
animate.model = \
animateColor.model = \
animateTransform.model = \
color_profile.model = \
cursor.model = \
mpath.model = \
set.model = \
view.model = sims.Elements(title, metadata, desc)
font.model = sims.Elements(title, missing_glyph, font_face, vkern, hkern, metadata, glyph, desc)
text.model = sims.ElementsOrText(a, animateMotion, set, textPath, tspan, title, animateColor, tref, animateTransform, altGlyph, animate, desc, metadata)
textPath.model = \
tspan.model = sims.ElementsOrText(a, set, title, tspan, animateColor, tref, altGlyph, animate, metadata, desc)
a.model = sims.ElementsOrText(defs, set, text, symbol, clipPath, use, animateColor, font_face, style, polyline, view, marker, path, line, font, color_profile, animate, rect, desc, a, cursor, animateMotion, polygon, g, svg, title, mask, altGlyphDef, ellipse, filter, script, switch, animateTransform, linearGradient, pattern, circle, radialGradient, image, metadata)
definition_src.model = \
font_face_format.model = \
font_face_name.model = \
glyphRef.model = \
hkern.model = \
vkern.model = sims.Empty()
altGlyph.model = \
desc.model = \
foreignObject.model = \
metadata.model = \
script.model = \
style.model = \
title.model = sims.NoElements()



# -*- coding: utf-8 -*-
# cython: language_level=3, always_allow_keywords=True

## Copyright 1999-2014 by LivingLogic AG, Bayreuth/Germany
## Copyright 1999-2014 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/xist/__init__.py for the license


"""
An XIST module that contains classes for describing XIST namespace modules.
From that info a skeleton implementation of the namespace module can be
generated.
"""


import sys, keyword, collections


__docformat__ = "reStructuredText"


def _simplify(value):
	"""
	Return a string whose value can be used as an intializer for an attribute
	value. (If the value is an :class:`int` strip the quotes)
	"""
	try:
		value = int(value)
	except ValueError:
		if '"' in value:
			value = repr(value)
		else:
			value = '"{}"'.format(repr(value)[1:-1])
	return value


class Error(Exception):
	pass


class RedefinedElementError(Error):
	def __init__(self, oldelement, newelement, duplicates):
		self.oldelement = oldelement
		self.newelement = newelement
		self.duplicates = duplicates

	def __str__(self):
		msg = "element named {0.oldelement.name} redefined".format(self)
		if self.duplicates == "allow":
			msgs = []
			if self.oldelement.attrs != self.newelement.attrs:
				# New element has different attributes
				removed = [attr.name for attr in self.oldelement.attrs.values() if attr.name not in self.newelement.attrs]
				if removed:
					msgs.append("attribute{} {} removed".format("s" if len(removed) > 1 else "", ", ".join(removed)))
				added = [attr.name for attr in self.newelement.attrs.values() if attr.name not in self.oldelement.attrs]
				if added:
					msgs.append("attribute{} {} added".format("s" if len(added) > 1 else "", ", ".join(added)))
			if msgs:
				msg += " ({})".format("; ".join(msgs))
		elif self.duplicates == "merge":
			incompatible = [attr.name for attr in self.oldelement.attrs if attr != self.newelement.attrs[attr.name]]
			msg += " (attribute{} {} incompatible)".format("s" if len(added) > 1 else "", ", ".join(incompatible))
		# else self.duplicates == "reject"
		return msg


class RedefinedProcInstError(Error):
	def __init__(self, oldprocinst, newprocinst, duplicates):
		self.oldprocinst = oldprocinst
		self.newprocinst = newprocinst
		self.duplicates = duplicates

	def __str__(self):
		return "procinst named {0.oldprocinst.named} redefined".format(self)


class RedefinedEntityError(Error):
	def __init__(self, oldentity, newentity, duplicates):
		self.oldentity = oldentity
		self.newentity = newentity
		self.duplicates = duplicates

	def __str__(self):
		return "entity named {0.oldentity.named} redefined".format(self)


class RedefinedCharRefError(Error):
	def __init__(self, oldcharref, newcharref, duplicates):
		self.oldcharref = oldcharref
		self.newcharref = newcharref
		self.duplicates = duplicates

	def __str__(self):
		return "charref named {0.oldcharref.named} with codepoint {0.oldcharref.codepoint} redefined with codepoint {0.newcharref.codepoint}".format(self)


def findname(basename, names):
	"""
	Return a new valid Python identifier based on :obj:`basename`, i.e. a name
	that is based on :obj:`basename`, a valid Python identifier and not in
	:obj:`names. Illegal characters in :obj:`basename` are replaced with ``_``
	and an ``_`` appendedwhen the name collides with a Python keyword.
	"""
	basename = "".join(c if "a" <= c <= "z" or "A" <= c <= "Z" or "0" <= c <= "9" or c == "_" else "_" for c in basename)
	testname = basename
	if keyword.iskeyword(basename):
		testname += "_"
	suffix = 2
	while testname in names:
		testname = "{0}{1}".format(basename, suffix)
		suffix += 1
	return testname


def _addlines(lines, newlines):
	l = len(newlines)
	if l == 0:
		lines[-1][1] += " pass"
	elif l == 1:
		lines[-1][1] += " {}".format(newlines[-1][1])
	else:
		lines.extend(newlines)


def _adddoc(lines, doc, level):
	if doc is not None:
		lines.append([level, '"""'])
		for line in doc.splitlines():
			lines.append([level, line])
		lines.append([level, '"""'])


class Module(object):
	def __init__(self, doc=None, defaultxmlns=None, indent="\t", encoding=None, defaults=False, model="fullonce", duplicates="reject"):
		self.doc = doc
		self.defaultxmlns = defaultxmlns
		self.indent = indent
		self.encoding = encoding if encoding is not None else sys.getdefaultencoding()
		self.defaults = defaults
		self.model = model
		self.duplicates = duplicates
		self.elements = collections.OrderedDict() # Maps (namespace name, element name) to ``xnd.Element``
		self.procinsts = collections.OrderedDict() # Maps pi target to ``xnd.ProcInst``
		self.entities = collections.OrderedDict() # Maps entity name to ``xnd.Entity``
		self.charrefs = collections.OrderedDict() # Maps charref name to ``xnd.CharRef``

	def __repr__(self):
		return "<{0.__class__.__module__}.{0.__class__.__name__} name={0.name!r} url={0.url!r} at {1:#x}>".format(self, id(self))

	def __iadd__(self, node):
		node._add(self)
		return self

	def __str__(self):
		lines = []
		self._aspy(lines, 0, set(), self)
		return "".join("{}{}\n".format(level*self.indent, text) for (level, text) in lines)

	def _findgroups(self):
		"""
		This method finds all attribute groups defined for any attribute. As an
		attribute group can be referenced multiple times (in fact that's the
		reason for the attribute groups existence), we have to make sure to list
		an attribute group only once. Furthermore the attribute groups should
		appear in the order in which they are referenced.
		"""
		# find all attribute groups defined for the attributes
		attrgroups = []
		attrgroupset = {}
		for node in self.elements.values():
			for attr in node.attrs.values():
				if attr.shared is not None and attr.shared not in attrgroupset:
					attrgroups.append(attr.shared)
					attrgroupset[attr.shared] = True
		return attrgroups

	def _aspy(self, lines, level, names, module):
		# Find all namespaces
		self._xmlnames = dict.fromkeys(e.xmlns for e in self.elements.values() if e.xmlns is not None)
		# Assign names to namespaces
		for xmlns in self._xmlnames:
			varname = findname("xmlns", names)
			self._xmlnames[xmlns] = varname
			names.add(varname)

		# assign names to all elements
		for child in self.elements.values():
			child.assignpyname(names)
			attrnames = set()
			for attr in child.attrs.values():
				attr.assignpyname(attrnames)

		# assign names to all processing instructions
		for child in self.procinsts.values():
			child.assignpyname(names)

		# assign names to all entitites
		for child in self.entities.values():
			child.assignpyname(names)

		# assign names to all character references
		for child in self.charrefs.values():
			child.assignpyname(names)

		# Assign names to attribute groups
		attrgroups = self._findgroups()
		for attrgroup in attrgroups:
			attrgroup.assignpyname(names)

		lines.append([level, "# -*- coding: {} -*-".format(module.encoding)])
		lines.append([0, ""])
		lines.append([0, ""])

		_adddoc(lines, self.doc, level)

		lines.append([level, "from ll.xist import xsc, sims"])

		# output namespace names
		if self._xmlnames:
			lines.append([0, ""])
			lines.append([0, ""])
			for (xmlns, varname) in self._xmlnames.items():
				lines.append([level, "{} = {!r}".format(varname, xmlns)])

		# output attribute groups
		for attrgroup in attrgroups:
			lines.append([0, ""])
			lines.append([0, ""])
			attrgroup._aspy(lines, level, names, module)

		# output elements, procinsts, entities and charref
		for nodetype in ("elements", "procinsts", "entities", "charrefs"):
			for node in getattr(self, nodetype).values():
				lines.append([0, ""])
				lines.append([0, ""])
				node._aspy(lines, level, names, module)

		# output schema information for the elements
		if module.model != "none":
			elswithschema = [node for node in self.elements.values() if not isinstance(node.modeltype, (bool, type(None)))]
			if elswithschema:
				lines.append([0, ""])
				lines.append([0, ""])
				newlines = []
				for node in elswithschema:
					modelargs = []
					if node.modelargs:
						for arg in node.modelargs:
							if isinstance(arg, Element):
								arg = arg.pyname
							modelargs.append(arg)
					newlines.append(("{}.model".format(node.pyname), "{}({})".format(node.modeltype, ", ".join(modelargs))))
				if self.model in ("simple", "fullall"):
					for line in newlines:
						lines.append([0, "{} = {}".format(*line)])
				elif self.model == "fullonce":
					newlines.sort(key=lambda l: l[1])
					for (i, line) in enumerate(newlines):
						(var, code) = line
						if i != len(newlines)-1 and code == newlines[i+1][1]:
							code = "\\"
						lines.append([0, "{} = {}".format(var, code)])
				else:
					raise ValueError("unknown sims mode {!r}".format(self.model))

	def shareattrs(self, all):
		# collect all identical attributes into lists
		identicalattrs = collections.OrderedDict()
		for node in self.elements.values():
			for attr in node.attrs.values():
				if attr.shared is None: # skip attributes that are already shared
					ident = attr.ident()
					try:
						attrs = identicalattrs[ident]
					except KeyError:
						attrs = identicalattrs[ident] = []
					attrs.append(attr)
		for (ident, attrs) in identicalattrs.items():
			# if the attribute appears more than once (or all attributes should be shared), define a group for it
			if all or len(attrs) > 1:
				group = AttrGroup(attrs[0].name)(attrs[0])
				for attr in attrs:
					attr.share(group)


class Named(object):
	def __init__(self, name):
		self.name = name
		self.pyname = None

	def __ne__(self, other):
		return not self == other

	def assignpyname(self, names, name=None):
		"""
		Assign a Python identifier to :obj:`self` (using either :obj:`name` or
		:obj:`self.name`). This uses :func:`findname` to create a valid Python
		identifier that is not in :obj:`names`.
		"""
		if name is None:
			name = self.name
		name = findname(name, names)
		self.pyname = name
		names.add(name)


class Element(Named):
	def __init__(self, xmlns, name, modeltype=None, modelargs=None, doc=None):
		Named.__init__(self, name)
		self.xmlns = xmlns
		self.attrs = collections.OrderedDict()
		self.modeltype = modeltype
		self.modelargs = modelargs
		self.doc = doc

	def __repr__(self):
		return "<{0.__class__.__module__}.{0.__class__.__name__} name={0.name!r} xmlns={0.xmlns!r} attrs={0.attrs!r} at {1:#x}>".format(self, id(self))

	def __eq__(self, other):
		# Don't compare the models
		return type(other) is Element and self.name == other.name and self.xmlns == other.xmlns and self.attrs == other.attrs

	def __hash__(self):
		return hash(self.name) ^ hash(self.xmlns) # don't include the attributes in the hash value, as they're unhashable

	def __iadd__(self, attr):
		self.attrs[attr.name] = attr
		return self

	def _add(self, module):
		key = (self.xmlns, self.name)
		if key in module.elements:
			oldelement = module.elements[key]
			if module.duplicates == "reject":
				raise RedefinedElementError(oldelement, self, duplicates=module.duplicates)
			elif module.duplicates == "allow":
				if module.elements[self.name] != self:
					raise RedefinedElementError(oldelement, self, duplicates=module.duplicates)
			else: # if duplicates == "merge"
				for attr in self.attrs.values():
					if attr.name in self.attrs:
						if attr != self.attrs[attr.name]:
							raise RedefinedElementError(oldelement, self, duplicates=module.duplicates)
					else:
						self.attrs[attr.name] = attr
		else:
			module.elements[key] = self
		if self.xmlns is None:
			self.xmlns = module.defaultxmlns

	def _aspy(self, lines, level, names, module):
		lines.append([level, "class {}(xsc.Element):".format(self.pyname)])
		newlines = []
		_adddoc(newlines, self.doc, level+1)
		if self.xmlns is not None:
			try:
				xmlns = module._xmlnames[self.xmlns] # Use variable name
			except KeyError:
				xmlns = _simplify(self.xmlns) # Use literal
			newlines.append([level+1, "xmlns = {}".format(xmlns)])
		if self.pyname != self.name:
			newlines.append([level+1, "xmlname = {}".format(_simplify(self.name))])
		# only output model if it is a bool, otherwise it might reference other element,
		# in which case this is done after all element classes have been defined
		if isinstance(self.modeltype, bool):
			newlines.append([level+1, "model = {!r}".format(self.modeltype)])

		if self.attrs:
			# find the attribute groups our elements are in
			# this means we don't have to define these attributes ourselves, but have to derive from the attribute group
			groups = []
			groupset = {}
			nogroup = []
			for attr in self.attrs.values():
				if attr.shared is not None:
					if tuple(attr.shared.attrs) not in groupset:
						groups.append(attr.shared)
				else:
					nogroup.append(attr)
			if groups:
				base = ", ".join(group.pyname for group in groups)
			else:
				base = "xsc.Element.Attrs"
			newlines.append([level+1, "class Attrs({}):".format(base)])
			if nogroup:
				localnames = []
				for attr in nogroup:
					attr._aspy(newlines, level+2, localnames, module)
			else:
				newlines.append([level+2, "pass"])
		_addlines(lines, newlines)


class AttrGroup(Named):
	id = 0

	def __init__(self, name):
		if name is None:
			name = "attrgroup_{}".format(self.__class__.id)
			self.__class__.id += 1
		Named.__init__(self, name)
		self.attrs = []

	def __call__(self, *content):
		self.attrs.extend(content)
		return self

	def _aspy(self, lines, level, names, module):
		lines.append([level, "class {}(xsc.Element.Attrs):".format(self.pyname)])
		localnames = []
		for attr in self.attrs:
			attr._aspy(lines, level+1, localnames, module)


class Attr(Named):
	def __init__(self, name, type, required=False, default=None, values=None, doc=None):
		Named.__init__(self, name)
		self.doc = doc
		self.type = type
		self.required = required
		self.default = default
		self.values = tuple(values) if values is not None else ()
		self.shared = None # if this attribute is part of a group ``shared`` will point to the group

	def __repr__(self):
		return "<{0.__class__.__module__}.{0.__class__.__name__} name={0.name!r} type={0.type!r} at {1:#x}>".format(self, id(self))

	def __eq__(self, other):
		return type(other) is Attr and self.name == other.name and self.type == other.type and self.required == other.required and self.default == other.default and self.values == other.values

	def __hash__(self):
		return hash(self.name) ^ hash(self.type) ^ hash(self.required) ^ hash(self.default) ^ hash(self.values)

	def _aspy(self, lines, level, names, module):
		name = self.name
		if isinstance(self.type, type):
			basename = "{0.type.__module__}.{0.type.__name__}".format(self)
		else:
			basename = self.type
		if basename.startswith("ll.xist.xsc."):
			basename = basename[8:]
		lines.append([level, "class {}({}):".format(self.pyname, basename)])
		newlines = []
		_adddoc(newlines, self.doc, level+1)
		if self.pyname != self.name:
			newlines.append([level+1, "xmlname = {}".format(_simplify(self.name))])
		if self.values:
			values = "({})".format(", ".join(str(_simplify(value)) for value in self.values))
			newlines.append([level+1, "values = {}".format(values)])
		if self.default and module.defaults:
			newlines.append([level+1, "default = {}".format(_simplify(self.default))])
		if self.required:
			newlines.append([level+1, "required = True"])
		_addlines(lines, newlines)

	def share(self, group):
		assert self.shared is None, "cannot share attr {!r} twice".format(self)
		self.shared = group

	def ident(self):
		return (self.name, self.type, self.required, self.default, tuple(self.values))


class ProcInst(Named):
	def __init__(self, name, doc=None):
		Named.__init__(self, name)
		self.doc = doc

	def __repr__(self):
		return "<{0.__class__.__module__}.{0.__class__.__name__} name={0.name!r} at {1:#x}>".format(self, id(self))

	def _add(self, module):
		if self.name in module.procinsts:
			if module.duplicates == "reject":
				raise RedefinedProcInstError(module.procinsts[self.name], self, module.duplicates)
		else:
			module.procinsts[self.name] = self

	def _aspy(self, lines, level, names, module):
		lines.append([level, "class {}(xsc.ProcInst):".format(self.pyname)])
		newlines = []
		_adddoc(newlines, self.doc, level+1)
		if self.pyname != self.name:
			newlines.append([level+1, "xmlname = {}".format(_simplify(self.name))])
		_addlines(lines, newlines)


class Entity(Named):
	def __init__(self, name, doc=None):
		Named.__init__(self, name)
		self.doc = doc

	def __repr__(self):
		return "<{0.__class__.__module__}.{0.__class__.__name__} name={0.name!r} at {1:#x}>".format(self, id(self))

	def __eq__(self, other):
		return type(other) is CharRef and self.name == other.name

	def _add(self, module):
		if self.name in module.entities:
			if module.duplicates == "reject":
				raise RedefinedEntityError(module.entities[self.name], self, module.duplicates)
		else:
			module.entities[self.name] = self

	def _aspy(self, lines, level, names, module):
		lines.append([level, "class {}(xsc.Entity):".format(self.pyname)])
		newlines = []
		_adddoc(newlines, self.doc, level+1)
		if self.pyname != self.name:
			newlines.append([level+1, "xmlname = {}".format(_simplify(self.name))])
		_addlines(lines, newlines)


class CharRef(Entity):
	def __init__(self, name, codepoint, doc=None):
		Entity.__init__(self, name, doc)
		self.codepoint = codepoint

	def __repr__(self):
		return "<{0.__class__.__module__}.{0.__class__.__name__} name={0.name!r} codepoint={0.codepoint:#x} at {1:#x}>".format(self, id(self))

	def __eq__(self, other):
		return type(other) is CharRef and self.name == other.name and self.codepoint == other.codepoint

	def _add(self, module):
		if self.name in module.charrefs:
			if module.duplicates == "reject":
				oldcharref = module.charrefs[self.name]
				raise RedefinedCharRefError(oldcharref, self, module.duplicates)
			else: # duplicates in ("allow", "merge"):
				oldcharref = module.charrefs[self.name]
				if oldcharref != self:
					raise RedefinedCharRefError(oldcharref, self, module.duplicates)
		else:
			module.charrefs[self.name] = self

	def _aspy(self, lines, level, names, module):
		lines.append([level, "class {}(xsc.CharRef):".format(self.pyname)])
		newlines = []
		_adddoc(newlines, self.doc, level+1)
		if self.pyname != self.name:
			newlines.append([level+1, "xmlname = {}".format(_simplify(self.name))])
		if self.codepoint > 0xffff:
			fmt = "#010x"
		elif self.codepoint > 0xff:
			fmt = "#06x"
		else:
			fmt = "#02x"
		newlines.append([level+1, "codepoint = {0:{1}}".format(self.codepoint, fmt)])
		_addlines(lines, newlines)



#!/usr/local/bin/python
# -*- coding: utf-8 -*-
# cython: language_level=3, always_allow_keywords=True

## Copyright 2007-2014 by LivingLogic AG, Bayreuth/Germany.
## Copyright 2007-2014 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/xist/__init__.py for the license


"""
Purpose
-------

``uhpp`` is a script for pretty printing HTML files. It is URL-enabled, so you
can specify local file names and URLs (and remote files via ``ssh`` URLs).


Options
-------

``uhpp`` supports the following options:

	``urls``
		Zero or more URLs to be printed. If no URL is given, stdin is read.

	``-v``, ``--verbose`` : ``false``, ``no``, ``0``, ``true``, ``yes`` or ``1``
		Output parse warnings?

	``-c``, ``--compact`` : ``false``, ``no``, ``0``, ``true``, ``yes`` or ``1``
		Compact HTML before printing (i.e. remove whitespace nodes)?


Examples
--------

Pretty print stdin::

	$ cat foo.html | uhpp

Pretty print a local HTML file::

	$ uhpp foo.html

Pretty print a remote HTML file::

	$ uhpp ssh://user@www.example.org/~/foo.html
"""


import sys, argparse

from ll import misc, url
from ll.xist import xsc, parse
from ll.xist.ns import xml, html

__docformat__ = "reStructuredText"


def main(args=None):
	def printone(u):
		source = parse.URL(u) if isinstance(u, url.URL) else parse.Stream(u)
		node = parse.tree(source, parse.Tidy(), parse.NS(html), parse.Node(base="", pool=xsc.Pool(html, xml)))
		if args.compact:
			node = node.normalized().compacted()
		node = node.pretty()
		print(node.string(encoding=sys.stdout.encoding))

	p = argparse.ArgumentParser(description="pretty print HTML files", epilog="For more info see http://www.livinglogic.de/Python/xist/scripts/uhpp.html")
	p.add_argument("urls", metavar="url", help="URLs to be pretty printed", nargs="*", type=url.URL)
	p.add_argument("-v", "--verbose", dest="verbose", help="Ouput parse warnings?", action=misc.FlagAction, default=False)
	p.add_argument("-c", "--compact", dest="compact", help="Compact HTML before pretty printing (default: %(default)s)", action=misc.FlagAction, default=False)

	args = p.parse_args(args)
	if not args.verbose:
		import warnings
		warnings.simplefilter("ignore", category=xsc.Warning)
	with url.Context():
		if args.urls:
			for u in args.urls:
				printone(u)
		else:
			printone(sys.stdin.buffer)


if __name__ == "__main__":
	sys.exit(main())



#! /usr/bin/env/python
# -*- coding: utf-8 -*-
# cython: language_level=3, always_allow_keywords=True

## Copyright 2014 by LivingLogic AG, Bayreuth/Germany.
## Copyright 2014 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/xist/__init__.py for the license


import io, os, tempfile

import cx_Oracle

import pytest

from ll import orasql, oradd


dbname = os.environ.get("LL_ORASQL_TEST_CONNECT") # Need a connectstring as environment var


def commands():
	yield dict(
		type="sql",
		sql="""
			create table oradd_test_table(
				odtt_id integer
			)
		"""
	)

	yield dict(
		type="sql",
		sql="""
			create or replace procedure oradd_test_procedure(
				p_odtt_id in out integer
			)
			as
			begin
				if p_odtt_id is null then
					p_odtt_id := 1;
				end if;
				insert into oradd_test_table (odtt_id) values (p_odtt_id);
				p_odtt_id := p_odtt_id + 100;
			end;
		"""
	)

	yield dict(
		type="sql",
		sql="""
			create sequence oradd_test_sequence
				minvalue 10
				maxvalue 9999999999999999999999999999
				start with 30
				increment by 10
				cache 20
		"""
	)

	yield dict(
		type="procedure",
		name="oradd_test_procedure",
		args=dict(
			p_odtt_id=oradd.var("odtt_1"),
		)
	)

	yield dict(
		type="procedure",
		name="oradd_test_procedure",
		args=dict(
			p_odtt_id=oradd.var("odtt_1"),
		)
	)

	yield dict(
		type="resetsequence",
		sequence="oradd_test_sequence",
		table="oradd_test_table",
		field="odtt_id"
	)

	yield dict(
		type="sql",
		sql="begin :filename_file := 'gurk_file.txt'; end;",
		args=dict(
			filename_file=oradd.var("filename_file", str),
		)
	)

	yield dict(
		type="sql",
		sql="begin :filename_scp := 'gurk_scp.txt'; end;",
		args=dict(
			filename_scp=oradd.var("filename_scp", str),
		)
	)

	yield dict(
		type="file",
		name="{filename_file}",
		content=b"gurk_file",
		mode=0o644,
	)

	yield dict(
		type="scp",
		name="{filename_scp}",
		content=b"gurk_scp",
	)


def execute_commands(commands, tmpdir):
	s = io.StringIO()

	for command in commands:
		print(repr(command), file=s)

	with tempfile.NamedTemporaryFile(delete=False) as f:
		f.write(s.getvalue().encode("utf-8"))
		tempname = f.name

	try:
		oradd.main([dbname, tempname, "-v3", "--scpdirectory", tmpdir, "--filedirectory", tmpdir])
	finally:
		os.remove(tempname)


def cleanup():
	with orasql.connect(dbname) as db:
		c = db.cursor()
		try:
			c.execute("drop table oradd_test_table")
		except cx_Oracle.DatabaseError:
			pass
		try:
			c.execute("drop sequence oradd_test_sequence")
		except cx_Oracle.DatabaseError:
			pass


@pytest.mark.db
def test_oradd(tmpdir):
	cleanup()

	execute_commands(commands(), "{}/".format(tmpdir))

	with orasql.connect(dbname) as db:
		c = db.cursor()
		c.execute("select odtt_id from oradd_test_table order by odtt_id")
		data = [int(r.odtt_id) for r in c]
		assert data == [1, 101]
		c.execute("select oradd_test_sequence.nextval as nv from dual")
		data = c.fetchone().nv
		assert data == 111

	f = tmpdir.join("gurk_file.txt")
	assert f.read() == "gurk_file"
	stat = os.stat(str(f))
	assert stat.st_mode & 0o777 == 0o644

	f2 = tmpdir.join("gurk_scp.txt")
	assert f2.read() == "gurk_scp"

	cleanup()
